@@ -9,7 +9,8 @@ from schematic.schemas.explorer import SchemaExplorer
 from schematic.utils.io_utils import load_json
 from schematic.utils.cli_utils import query_dict
 from schematic.utils.schema_utils import load_schema_into_networkx
-from schematic.utils.validate_utils import validate_schema
+from schematic.utils.validate_utils import validate_schema, rule_in_rule_list
+
 from schematic import CONFIG
 
 logger = logging.getLogger(__name__)
@@ -176,12 +177,14 @@ class SchemaGenerator(object):
 
         # get a list of required component nodes
         req_components = self.get_component_requirements(source_component)
-        
+
         # get the schema graph
         mm_graph = self.se.get_nx_schema()
 
         # get the subgraph induced on required component nodes
-        req_components_graph = self.get_subgraph_by_edge_type(mm_graph, self.requires_component_relationship).subgraph(req_components)
+        req_components_graph = self.get_subgraph_by_edge_type(
+            mm_graph, self.requires_component_relationship
+        ).subgraph(req_components)
 
         return req_components_graph
 
@@ -531,8 +534,7 @@ class SchemaGenerator(object):
                         if node_validation_rules:
                             # if this node has extra validation rules process them
                             # TODO: abstract this into its own validation rule constructor/generator module/class
-
-                            if "list" in node_validation_rules:
+                            if rule_in_rule_list("list", node_validation_rules):
                                 # if this node can be mapped to a list of nodes
                                 # set its schema accordingly
                                 schema_valid_vals = self.get_array_schema(
@@ -574,20 +576,18 @@ class SchemaGenerator(object):
                                     )
 
                                     if node_validation_rules:
-                                        if "list" in node_validation_rules:
+                                        if rule_in_rule_list("list", node_validation_rules):
                                             # TODO: get_range_schema and get_range_schema have similar behavior - combine in one module
-                                            dependency_properties = (
-                                                self.get_array_schema(
-                                                    node_range_d,
-                                                    node_display_name,
-                                                    blank=not node_required,
-                                                )
+                                            dependency_properties = self.get_array_schema(
+                                                node_range_d,
+                                                node_display_name,
+                                                blank=not node_required,
                                             )
 
                                 else:
                                     if node_required:
-                                        dependency_properties = (
-                                            self.get_non_blank_schema(node_display_name)
+                                        dependency_properties = self.get_non_blank_schema(
+                                            node_display_name
                                         )
                                     else:
                                         dependency_properties = {node_display_name: {}}
@@ -613,7 +613,9 @@ class SchemaGenerator(object):
                             )
 
                             if node_validation_rules:
-                                if "list" in node_validation_rules:
+                                # If there are valid values AND they are expected to be a list,
+                                # reformat the Valid Values.
+                                if rule_in_rule_list("list", node_validation_rules):
                                     schema_valid_vals = self.get_array_schema(
                                         node_range_d, node_display_name, blank=False
                                     )
@@ -635,7 +637,7 @@ class SchemaGenerator(object):
                             )
 
                             if node_validation_rules:
-                                if "list" in node_validation_rules:
+                                if rule_in_rule_list("list", node_validation_rules):
                                     schema_valid_vals = self.get_array_schema(
                                         node_range_d, node_display_name, blank=True
                                     )
@@ -706,7 +708,9 @@ class SchemaGenerator(object):
                 "nested key in the configuration: (model > input > log_location)."
             )
         else:
-            os.makedirs(os.path.dirname(json_schema_log_file), exist_ok=True)
+            json_schema_dirname = os.path.dirname(json_schema_log_file)
+            if json_schema_dirname != '':
+                os.makedirs(json_schema_dirname, exist_ok=True)
             with open(json_schema_log_file, "w") as js_f:
                 json.dump(json_schema, js_f, indent=2)
 
